import{P as Popover,T as Tooltip}from"./bootstrap.esm-BOBBXxos.js";import{$}from"./jquery-BALIvAru.js";import{N as createApp}from"./vue-CSzJdJby.js";import{_ as _sfc_main}from"./Error-BXO_ADNF.js";import"./highlight.js-CWp04Ajj.js";window.errorApp=createApp({components:{error:_sfc_main}}).mount("#errorApp");window.clearError=function(){window.errorApp.$refs.error.clear()};window.showError=function(e){typeof e=="string"?window.errorApp.$refs.error.$refs.error.setHTML(e):window.errorApp.$refs.error.fromXHR(e)};$(()=>{var url=window.location,element=$("ul.nav a").filter(function(){return this.href==url||url.href.indexOf(this.href)==0}).addClass("active").parent().parent().addClass("in").parent();element.is("li")&&element.addClass("active"),eval(localStorage.getItem("sidebarCollapsed"))&&(document.body.className+="sidebar-collapsed"),$("#menu-collapse").click(function(e){e.preventDefault(),$(document.body).toggleClass("sidebar-collapsed"),localStorage.setItem("sidebarCollapsed",$(document.body).hasClass("sidebar-collapsed")),window.dispatchEvent(new Event("resize"))});const popoverTriggerList=document.querySelectorAll('[data-bs-toggle="popover"]');[...popoverTriggerList].map(e=>new Popover(e,{sanitize:!1}));const tooltipTriggerList=document.querySelectorAll('[data-bs-toggle="tooltip"]');[...tooltipTriggerList].map(e=>new Tooltip(e,{sanitize:!1}))});
