{% extends "base.html" %}

{% block title %}temBoard / About{% endblock %}

{% block head %}
  {% for link in vitejs.css_links_for('settings.about.js') %}
    {{ link | safe}}
  {% endfor %}
{% endblock %}

{% block content %}

<div id="app">
  <about
    :infos='{{ infos | tojson }}'
    temboard_version="{{ temboard_version }}">
  </about>
</div>
<script type="module" src="{{ vitejs.url_for('settings.about.js') }}"></script>
{% endblock %}
