{% extends "base.html" %}

{% block title %}{{ super() }}/ Authentication required{% endblock %}

{% block content %}
<div class="row justify-content-center">
  <div class="col col-xl-6 col-10">
    <div class="alert alert-danger alert-restricted" role="alert">
      <h4><i class="fa fa-ban fa-fw"></i> Authentication required <small></small></h4>
      <p>Go to <a href="/login">login page</a> to authenticate.</p>
    </div>
  </div>
</div>
{% endblock %}
