import{$ as t}from"./jquery-BALIvAru.js";import"./highlight.js-CWp04Ajj.js";t("#sendEmailForm").on("submit",function(e){e.preventDefault(),o()});t("#sendSmsForm").on("submit",function(e){e.preventDefault(),s()});function o(){clearError(),t.ajax({url:"/json/test_email",type:"post",contentType:"application/json",data:JSON.stringify({email:t("#inputTestEmail").val()}),success:function(e){var n="Test email successfully sent";n+=`
If the email is not received, please have a look at your SMTP server logs`,alert(n)},error:function(e){console.log("error",e),showError(e)}})}function s(){clearError(),t.ajax({url:"/json/test_sms",type:"post",contentType:"application/json",data:JSON.stringify({phone:t("#inputTestPhone").val()}),success:function(e){alert("Test SMS sent")},error:showError})}
