import{p as m,s as u,o as _,e as h,z as e,g as p,f as d,r as f}from"./vue-CSzJdJby.js";import{M as v}from"./bootstrap.esm-BOBBXxos.js";import"./jquery-BALIvAru.js";const b=["id"],g={class:"modal-dialog",role:"document"},w={class:"modal-content"},x={class:"modal-header"},M={class:"modal-title"},k=e("button",{type:"button",class:"btn-close","data-bs-dismiss":"modal","aria-label":"Close"},null,-1),L={__name:"ModalDialog",props:["id","title"],emits:["opening","closed"],setup(s,{expose:l,emit:i}){const a=i,o=m(null);let t=null;function n(){t.show()}function c(){t.hide()}return l({hide:c,show:n}),u(()=>{t=new v(o.value),o.value.addEventListener("show.bs.modal",()=>{a("opening")}),o.value.addEventListener("hidden.bs.modal",()=>{a("closed")})}),(r,y)=>(_(),h("div",{id:s.id,ref_key:"root",ref:o,class:"modal fade",role:"dialog",tabindex:"-1","aria-hidden":"true"},[e("div",g,[e("div",w,[e("div",x,[e("h4",M,p(s.title),1),d(),k]),d(),f(r.$slots,"default")])])],8,b))}};export{L as _};
