{% extends "base.html" %}

{% block title %}{{ super() }}/ Login{% endblock %}

{% block content %}
<div class="row justify-content-center">
  <div class="col-12 col-md-6 col-lg-4">
    <div class="card mt-5">
      <div class="card-header bg-primary text-center">
        <img src="/images/temboard-150x32-w.png" />
      </div>
      <div class="card-body">
      <form onsubmit="login(event);">
        <div class="mb-3">
          <label for="inputUsername" class="form-label">Username</label>
          <input type="text" name="username" class="form-control" id="inputUsername" placeholder="Username" autofocus>
        </div>
        <div class="mb-3">
          <label for="inputPassword" class="form-label">Password</label>
          <input type="password" name="password" class="form-control" id="inputPassword" placeholder="Password">
        </div>
        <div class="text-center">
          <button type="submit" class="btn btn-primary">Login</button>
        </div>
        </form>
      </div>
    </div>
  </div>
</div>
<script type="module" src="{{ vitejs.url_for('login.js') }}"></script>
{% endblock %}
