{% extends "base.html" %}

{% block title %}{{ super() }}/ Not Found{% endblock %}

{% block content %}
<div class="row justify-content-center">
  <div class="col col-xl-6 col-10">
    <div class="alert alert-danger alert-restricted" role="alert">
      <h4><i class="fa fa-ban fa-fw"></i> Page not Found <small></small></h4>
      <p>The page you are looking for might have been removed or had its name changed.</p>
    </div>
  </div>
</div>
{% endblock %}
