{% extends "base.html" %}

{% block title %}{{ super() }}/ Error{% endblock %}

{% block content %}
<div class="row justify-content-center">
  <div class="col col-xl-6 col-10">
    <div class="alert alert-danger" role="alert">
      <h4><i class="fa fa-ban fa-fw"></i> Internal Error</h4>
      <p>The following unhandled error occured:</p>
      <pre>{{ message }}</pre>
    </div>
  </div>
</div>
{% endblock %}
