import{$ as r}from"./jquery-BALIvAru.js";import"./highlight.js-CWp04Ajj.js";function e(n){n.preventDefault(),r.ajax({url:"/json/login",type:"post",data:JSON.stringify({username:r("#inputUsername").val(),password:r("#inputPassword").val()}),headers:{"Content-Type":"application/json"},success:function(o){window.location.href=document.referrer?document.referrer:"/"},error:function(o){console.log("error",o),showError(o)}})}window.login=e;
