import{e as i,z as t,f as e,g as n,k as _,w as d,j as r,F as b,A as h,S as p,o,n as g,b as f,h as u,N as y}from"./vue-CSzJdJby.js";import{a as x,U as w}from"./index-ncPPdFzC.js";import"./index-C25aB_MX.js";const k="/static/images/heron.png",v={class:"row"},C={class:"col-12",id:"content",style:{"font-size":"130%"}},B={style:{"max-width":"800px"},class:"mx-auto w-100 w-md-75"},N=t("div",{class:"row justify-content-center"},[t("img",{class:"m-4 w-50 w-xl-25",src:k})],-1),A={class:"row"},U={class:"col-12 text-center"},V={style:{"font-size":"3rem",color:"#d65a16"},class:"fw-bold"},z=p(`<div class="btn-group mx-auto justify-content-center"><a href="https://labs.dalibo.com/temboard" class="badge text-bg-light mx-1" target="_blank"><i class="fa fa-globe"></i> Official website
              </a> <a href="https://temboard.readthedocs.io/en/latest/" class="badge text-bg-light mx-1" target="_blank"><i class="fa fa-book"></i> Documentation
              </a> <a href="https://github.com/dalibo/temboard" class="badge text-bg-light mx-1" target="_blank"><i class="fa fa-github"></i> GitHub
              </a> <a href="https://github.com/dalibo/temboard/issues/new" class="badge text-bg-light mx-1" target="_blank"><i class="fa fa-bug"></i> New issue
              </a></div>`,1),S={style:{position:"relative"}},j=["onClick"],T=t("i",{class:"fa fa-copy"},null,-1),D={class:"row mt-2 m-1"},E={id:"metadata"},F=t("p",{style:{display:"none"}},"About temBoard",-1),G={class:"mb-1"},H=["data-value"],I=["id"],L={key:0,style:{display:"none"}},M=t("strong",null,"Start time:",-1),O={__name:"About",props:["infos","temboard_version"],setup(c){function m(){return document.getElementById("metadata").innerText}return($,q)=>(o(),i("div",v,[t("div",C,[t("div",B,[N,e(),t("div",A,[t("div",U,[t("h1",V,"temBoard "+n(c.temboard_version),1),e(),z])]),e(),t("div",S,[_(r(x),{legacy:!0},{default:d(({copy:s,copied:a})=>[t("button",{id:"buttonCopy",class:"btn btn-light",style:{position:"absolute",top:"0",right:"0"},title:"Copy to clipboard",onClick:l=>s(m())},[T,e(" "+n(a?"Copied":"Copy"),1)],8,j)]),_:1})]),e(),t("div",D,[t("div",E,[F,e(),(o(!0),i(b,null,h(c.infos,(s,a)=>(o(),i("div",{key:a},[t("p",G,[t("strong",null,n(a)+":",1),e(),s===!0||s===!1?(o(),i("i",{key:0,class:g(["fa",{"fa-check":s,"fa-times":!s}]),"data-value":s},null,10,H)):a==="Uptime"?(o(),f(r(w),{key:1,time:s},{default:d(({timeAgo:l})=>[e(n(l),1)]),_:2},1032,["time"])):(o(),i("span",{key:2,id:a},n(s),9,I))]),e(),a==="Uptime"?(o(),i("p",L,[M,e(),t("span",null,"("+n(s)+")",1)])):u("",!0)]))),128))])])])])]))}};y({components:{about:O}}).mount("#app");
