{% extends ../../../templates/base.html %}

{% block title %}temBoard / {{instance}} / Monitoring{% end %}

{% block content %}
<div id="app">
  <alerting-checks />
</div>

<script type="module" src="{{ vitejs.url_for('alerting.checks.js') }}"></script>
<script>
var apiUrl = "/server/{{instance.agent_address}}/{{instance.agent_port}}/alerting"
</script>

{% end %}
