import{o as n,e as d,z as t,C as e,f as c}from"./vue-CSzJdJby.js";const i=["data-bs-content"],g={class:"progress rounded-0"},h=["data-bs-content"],w={class:"progress rounded-0"},m=["data-bs-content"],v={__name:"SizeDistributionBar",props:["height","total","cat1","cat1raw","cat1label","cat1bis","cat1bisraw","cat1bislabel","cat2","cat2raw","cat2label","cat2","cat2bis","cat2bisraw","cat2bislabel","cat3","cat3raw","cat3label"],setup(a){const o=a;function s(r,l){return`width: ${100*r/l}%`}function b(r){let l=`${o[r+"label"]}: ${o[r]}`;return o[r+"bis"]&&(l+=`<br><small class="d-inline-block text-body-secondary">Bloat: ${o[r+"bis"]}</small>`),l}return(r,l)=>(n(),d("div",{class:"progress border rounded-0",style:e({height:o.height+"px"})},[t("div",{class:"progress-bar",role:"progressbar",style:e(s(a.cat1raw,a.total)),"data-bs-toggle":"popover","data-bs-trigger":"hover","data-bs-placement":"bottom","data-bs-html":"true","data-bs-content":b("cat1")},[t("div",g,[t("div",{class:"progress-bar bg-cat1",role:"progressbar",style:e(s(a.cat1raw-a.cat1bisraw,a.cat1raw))},null,4),c(),t("div",{class:"progress-bar bg-cat1 progress-bar-striped-small",role:"progressbar",style:e(s(a.cat1bisraw,a.cat1raw))},null,4)])],12,i),c(),t("div",{class:"progress-bar",role:"progressbar",style:e(s(a.cat2raw,a.total)),"data-bs-toggle":"popover","data-bs-trigger":"hover","data-bs-placement":"bottom","data-bs-html":"true","data-bs-content":b("cat2")},[t("div",w,[t("div",{class:"progress-bar bg-cat2",role:"progressbar",style:e(s(a.cat2raw-a.cat2bisraw,a.cat2raw))},null,4),c(),t("div",{class:"progress-bar bg-cat2 progress-bar-striped-small",role:"progressbar",style:e(s(a.cat2bisraw,a.cat2raw))},null,4)])],12,h),c(),t("div",{class:"progress-bar bg-secondary",role:"progressbar",style:e(s(a.cat3raw,a.total)),"data-bs-toggle":"popover","data-bs-trigger":"hover","data-bs-placement":"bottom","data-bs-content":b("cat3")},null,12,m)],4))}};export{v as _};
