import{_ as M}from"./lodash-uc_7-CSk.js";const r=1e3,u=r*60,a=u*60,e=a*24;function l(t,p){let n=[];const d=Math.round((t-Math.floor(t))*1e3);t=Math.floor(t);const c=Math.trunc(t/e);n.push(c+" d"),t=t-c*e;const h=Math.trunc(t/a);n.push(h+" h"),t=t-h*a;const i=Math.trunc(t/u);n.push(i+" min"),t=t-i*u;const s=Math.trunc(t/r);if(n.push(s+" s"),t=t-s*r,n.push(t+" ms"),n.push(d+" µs"),n=M.dropWhile(n,o=>parseInt(o)==0),p){const o=parseInt(n[0])<3&&parseInt(n[1])!==0?2:1;n=n.slice(0,o)}return n.length?n.join(" "):"0 ms"}export{l as f};
