import{V as e,D as t}from"./datatables.net-vue3-B28JL-vi.js";import"./jquery-BALIvAru.js";import"./highlight.js-CWp04Ajj.js";import"./vue-CSzJdJby.js";e.use(t);new t("#tableNotifications",{order:[[0,"desc"]],pageLength:25});
