<ul class="nav nav-pills flex-column">
  <li class="nav-item">
    <a href="/settings/instances" class="nav-link"><i class="fa fa-fw fa-database"></i><span>Instances</span></a>
  </li>
  <li class="nav-item">
    <a href="/settings/environments" class="nav-link"><i class="fa fa-fw fa-folder"></i><span>Environments</span></a>
  </li>
  <li class="nav-item">
    <a href="/settings/users" class="nav-link"><i class="fa fa-fw fa-user"></i><span>Users</span></a>
  </li>
  <li class="nav-item">
    <a href="/settings/notifications" class="nav-link"><i class="fa fa-fw fa-paper-plane"></i><span>Alert notifications</span></a>
  </li>
  <li class="nav-item">
    <a href="/about" class="nav-link"> <i class="fa fa-fw fa-info"></i><span>About temBoard</span></a>
  </li>
</ul>
