{% extends base.html %}

{% block title %}temBoard{% end %}

{% block content %}
<div class="row">
  <div class="col-md-6 col-md-offset-3">
    <div class="alert alert-danger" role="alert">
      <h4><i class="fa fa-ban fa-fw"></i> Error</h4>
      <p>{{error}}</p>
      <br />
      <div class="text-center"><a class="btn btn-sm btn-danger" href="/">Back to home page</a></div>
    </div>
  </div>
</div>
{% end %}
